# -*- coding: utf-8 -*-

import os
import platform
import sys
import subprocess
from pathlib import Path
from typing import Callable, Dict, Union

__all__ = ('check', 'run', 'step')


class BGColors:
    """ Цветное форматирование текста """
    OK = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    END = '\033[0m'

    @classmethod
    def __format(cls, color, text):
        return '{}{}{}'.format(color, text, cls.END)

    @classmethod
    def success(cls, text: str):
        return cls.__format(cls.OK, text)

    @classmethod
    def error(cls, text: str):
        return cls.__format(cls.FAIL, text)


def run(command: str, *, silent: bool = True):
    """
    Запуск команды

    @param command: строка команды
    @param silent: режим тишины

    @return: True если успех, False иначе
    """
    params = {'stdout': subprocess.DEVNULL, 'stderr': subprocess.DEVNULL}
    result = subprocess.call(command, shell=True, **params if silent else {})
    return result == 0


def check(program: Path) -> Union[bool, str]:
    """
    Проверка системы

    @param program: путь к устанавливаемой программе

    @return: True - успех, строка - ошибка
    """

    if sys.version_info < (3, 5):
        return 'Для работы необходим Python v3.5 и выше.'

    if 'astra' not in platform.version().lower():
        return 'Система определяется не как AstraLinux.'

    if os.geteuid() != 0:
        return 'Для работы необходимы привилегии root.'

    if not (program.exists() and program.is_dir()):
        return 'Не найден каталог "{}" с программой для установки.'.format(program)

    return True


def step(title: str, function: Callable, arguments: Dict = None):
    """
    Шаг установки

    @param title: заголовок
    @param function: функция шага
    @param arguments: аргументы функции шага
    """
    print(title, '...', end=' ', flush=True)

    if arguments is None:
        arguments = {}

    result, error = function(**arguments), None
    if isinstance(result, str):
        error = result
    elif not result:
        error = 'FAIL'

    if error:
        print(BGColors.error(error))
        sys.exit(1)

    print(BGColors.success('OK'))
